from transformers import AutoTokenizer, AutoModelForCausalLM
import torch
from huggingface_hub import login
import json

# 登录 Hugging Face 账号（替换为你的访问令牌）
login(token="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")

# 模型信息
modelname = "meta-llama/Llama-2-7b-chat-hf"
device_id = 0
device = torch.device(f"cuda:{device_id}" if torch.cuda.is_available() else "cpu")
print(f"Using device: {device}")

# 加载分词器和模型
tokenizer = AutoTokenizer.from_pretrained(modelname)
model = AutoModelForCausalLM.from_pretrained(modelname)
model.to(device)

# 读取 JSON 文件
json_file_path = "/Users/shaw/PycharmProjects/math500/dataquestion/problemsenglish.json"  # 替换为你的 JSON 文件路径

# 定义一个函数处理 JSON 文件
def process_json(file_path):
    prompts = []
    with open(file_path, "r", encoding="utf-8") as file:
        data = json.load(file)  # 直接读取整个 JSON 文件
        for entry in data:
            problem = entry.get("problem", "")  # 获取每个条目的 'problem' 字段
            prompt = (
                "You are a math expert. Please solve the following math problem and provide a step-by-step solution:\n"
                f"Problem: {problem}\nSolution:"
            )
            prompts.append({"problem": problem, "prompt": prompt})
    return prompts

# 生成模型回答
def generate_solution(prompt):
    inputs = tokenizer(prompt, return_tensors="pt").to(device)
    with torch.no_grad():
        outputs = model.generate(**inputs, max_length=300, num_beams=5, early_stopping=True)
    generated_text = tokenizer.decode(outputs[0], skip_special_tokens=True)
    return generated_text

# 处理 JSON 文件并生成回答
prompts = process_json(json_file_path)
answers = []

for item in prompts:
    problem = item["problem"]
    prompt = item["prompt"]
    answer = generate_solution(prompt)  # 使用 'answer' 字段
    answers.append({"problem": problem, "answer": answer})
    print("Problem:\n", problem)
    print("Generated Answer:\n", answer)
    print("=" * 50)

# 保存输出为单个 JSON 文件
output_file_path = "output.json"
with open(output_file_path, "w", encoding="utf-8") as file:
    json.dump(answers, file, ensure_ascii=False, indent=4)

print(f"Generated answers have been saved to {output_file_path}")
